/******************************************************************************
*Copyright (C) 2003 Freescale Semiconductor, Inc.
*All Rights Reserved
*
* Filename:		DEMO9S08QG8.h
* Author:		r1aald
* Revision:		1.0a
*
* Description:	9S08QG8 Demo Board Header File
* Notes:		
*******************************************************************************/

#include <MC9S08QG8.h> /* include peripheral declarations */

static const unsigned char a_word_to_the_wise[] =
{
  "DEFCON 15 Badge by Joe Grand (Kingpin)     \
  Grand Idea Studio (www.grandideastudio.com) \
  Good luck with the badge hacking contest... \
  Find me for schwag and tips! \
  All is not L0pht..." 
};
  
// Defines

typedef enum // define the modes for the state machine
{
   SLEEP,
   SCROLL,
   CLOCK,
   RIDER, 
   SET_MSG,
   SET_CLOCK,
   SET_SPEED,
   SET_BOUNCE,
   SET_BLINK,
   POV
} state_type;


#define LOW   0
#define HIGH  1


// 74HC595 shift register (for LEDs)
#define NUMBER_OF_74595     3
#define REG_DATA      PTAD_PTAD2    // Output 
#define REG_STROBE    PTBD_PTBD7    // Output
#define REG_CLK       PTBD_PTBD6    // Output

#define U4  2
#define U5  1
#define U6  0
 
#define NUM_ROWS  19
            
// switches
#define SW1   PTAD_PTAD0
#define SW2   PTAD_PTAD1


/* function declarations */

// main.c
void write_expanded_outputs (unsigned char eo[NUMBER_OF_74595]);
void delay (unsigned int count);
void delay_ms(unsigned int ms);
void enable_scrolling (void);
void disable_scrolling (void); 
void change_state (void);

// led matrix routines
void clear_matrix(void);
void draw_buffer(unsigned char* buf);
void draw_rider(unsigned char start, const unsigned char* buf);
void LedStr (char *dataPtr);
void LedClock (void);

// called from MCUinit.c
void MCU_init(void); 
void KBI_ISR (void);
void MTIM_ISR (void);

#define CHAR_LENGTH 16
#define MSG_LENGTH  (CHAR_LENGTH * 5) + 41  // 5 bytes per character + 40 bytes blank + 1 end-of-message marker

// const unsigned char str_default[] = "I { DEFCON"; // default scrolling message
const unsigned char str_default[] = "Defcon 15";
#define MAGIC_PASSWORD "31337"
const unsigned char str_easteregg[] = "$ SUNSHINE $";

  
// Sorry, these arrays are totally inefficient, but it's the easiest way
// since we use the existing draw_buffer routine, which uses fb
// Each complete row of the array is a frame, each element in the array (left to right)
// corresponds to an individual row of LEDs, starting from the top. 
 
// DEFCON text
#define POV_MAX_FRAME   51
const char POV_buffer [POV_MAX_FRAME][NUM_ROWS] = 
{  
  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
  { 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x00 }, // D
  { 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x00 },
  { 0x0E, 0x0E, 0x0E, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0E, 0x0E, 0x0E, 0x00 },
  { 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x00 },
  { 0x00, 0x00, 0x00, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x00, 0x00, 0x00, 0x00 },
  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
  { 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x00 }, // E
  { 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x00 },
  { 0x0E, 0x0E, 0x0E, 0x00, 0x00, 0x00, 0x0E, 0x0E, 0x0E, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0E, 0x0E, 0x0E, 0x00 },
  { 0x0E, 0x0E, 0x0E, 0x00, 0x00, 0x00, 0x0E, 0x0E, 0x0E, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0E, 0x0E, 0x0E, 0x00 },
  { 0x0E, 0x0E, 0x0E, 0x00, 0x00, 0x00, 0x0E, 0x0E, 0x0E, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0E, 0x0E, 0x0E, 0x00 },
  { 0x0E, 0x0E, 0x0E, 0x00, 0x00, 0x00, 0x0E, 0x0E, 0x0E, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0E, 0x0E, 0x0E, 0x00 },
  { 0x0E, 0x0E, 0x0E, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0E, 0x0E, 0x0E, 0x00 },
  { 0x0E, 0x0E, 0x0E, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0E, 0x0E, 0x0E, 0x00 },
  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
  { 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x00 }, // F
  { 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x00 },
  { 0x0E, 0x0E, 0x0E, 0x00, 0x00, 0x00, 0x0E, 0x0E, 0x0E, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
  { 0x0E, 0x0E, 0x0E, 0x00, 0x00, 0x00, 0x0E, 0x0E, 0x0E, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
  { 0x0E, 0x0E, 0x0E, 0x00, 0x00, 0x00, 0x0E, 0x0E, 0x0E, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
  { 0x0E, 0x0E, 0x0E, 0x00, 0x00, 0x00, 0x0E, 0x0E, 0x0E, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
  { 0x0E, 0x0E, 0x0E, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
  { 0x00, 0x00, 0x00, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x00, 0x00, 0x00, 0x00 }, // C
  { 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x00 },
  { 0x0E, 0x0E, 0x0E, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0E, 0x0E, 0x0E, 0x00 },
  { 0x0E, 0x0E, 0x0E, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0E, 0x0E, 0x0E, 0x00 },
  { 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x00 },
  { 0x00, 0x00, 0x00, 0x0E, 0x0E, 0x0E, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0E, 0x0E, 0x0E, 0x00, 0x00, 0x00, 0x00 },
  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
  { 0x00, 0x00, 0x00, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x00, 0x00, 0x00, 0x00 }, // O
  { 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x00 },
  { 0x0E, 0x0E, 0x0E, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0E, 0x0E, 0x0E, 0x00 },
  { 0x0E, 0x0E, 0x0E, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0E, 0x0E, 0x0E, 0x00 },
  { 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x00 },
  { 0x00, 0x00, 0x00, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x00, 0x00, 0x00, 0x00 },
  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
  { 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x00 }, // N
  { 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x00 },
  { 0x00, 0x00, 0x00, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x00, 0x00, 0x00, 0x00 },
  { 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x00 },
  { 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x0E, 0x00 },
  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 }
};

// Floppy disk
/*#define POV_MAX_FRAME   19
const char POV_buffer [POV_MAX_FRAME][NUM_ROWS] = 
{  
  { 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b01110, 0b01110, 0b01110, 0b01110, 0b01110, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000 }, 
  { 0b00000, 0b00000, 0b00000, 0b00000, 0b01110, 0b01110, 0b01110, 0b01110, 0b01110, 0b01110, 0b01110, 0b01110, 0b01110, 0b01110, 0b01110, 0b00000, 0b00000, 0b00000, 0b00000 }, 
  { 0b00000, 0b00000, 0b00000, 0b01110, 0b01110, 0b01110, 0b01110, 0b01110, 0b01110, 0b01110, 0b01110, 0b01110, 0b01110, 0b01110, 0b01110, 0b01110, 0b00000, 0b00000, 0b00000 }, 
  { 0b00000, 0b00000, 0b01110, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b01110, 0b00000, 0b00000 }, 
  { 0b00000, 0b01110, 0b01110, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b01110, 0b01110, 0b00000 }, 
  { 0b00000, 0b01110, 0b01110, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b01110, 0b01110, 0b00000 }, 
  { 0b00000, 0b01110, 0b01110, 0b00000, 0b00000, 0b00000, 0b00000, 0b01110, 0b01110, 0b01110, 0b01110, 0b01110, 0b00000, 0b00000, 0b00000, 0b00000, 0b01110, 0b01110, 0b00000 }, 
  { 0b01110, 0b01110, 0b01110, 0b00000, 0b00000, 0b00000, 0b01110, 0b01110, 0b00000, 0b00000, 0b00000, 0b01110, 0b01110, 0b00000, 0b00000, 0b01110, 0b01110, 0b01110, 0b01110 }, 
  { 0b01110, 0b01110, 0b01110, 0b00000, 0b00000, 0b00000, 0b01110, 0b00000, 0b01110, 0b01110, 0b01110, 0b00000, 0b01110, 0b00000, 0b00000, 0b00000, 0b01110, 0b01110, 0b01110 }, 
  { 0b01110, 0b01110, 0b01110, 0b00000, 0b00000, 0b00000, 0b01110, 0b00000, 0b01110, 0b01110, 0b01110, 0b00000, 0b01110, 0b00000, 0b01110, 0b01110, 0b01110, 0b01110, 0b01110 }, 
  { 0b01110, 0b01110, 0b01110, 0b00000, 0b00000, 0b00000, 0b01110, 0b00000, 0b01110, 0b01110, 0b01110, 0b00000, 0b01110, 0b00000, 0b00000, 0b00000, 0b01110, 0b01110, 0b01110 }, 
  { 0b01110, 0b01110, 0b01110, 0b00000, 0b00000, 0b00000, 0b01110, 0b01110, 0b00000, 0b00000, 0b00000, 0b01110, 0b01110, 0b00000, 0b00000, 0b01110, 0b01110, 0b01110, 0b01110 }, 
  { 0b00000, 0b01110, 0b01110, 0b00000, 0b00000, 0b00000, 0b00000, 0b01110, 0b01110, 0b01110, 0b01110, 0b01110, 0b00000, 0b00000, 0b00000, 0b00000, 0b01110, 0b01110, 0b00000 }, 
  { 0b00000, 0b01110, 0b01110, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b01110, 0b00000, 0b00000, 0b01110, 0b01110, 0b00000 }, 
  { 0b00000, 0b01110, 0b01110, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b01110, 0b01110, 0b00000 }, 
  { 0b00000, 0b00000, 0b01110, 0b00000, 0b00000, 0b01110, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b01110, 0b00000, 0b00000 }, 
  { 0b00000, 0b00000, 0b00000, 0b01110, 0b01110, 0b01110, 0b01110, 0b01110, 0b01110, 0b01110, 0b01110, 0b01110, 0b01110, 0b01110, 0b01110, 0b01110, 0b00000, 0b00000, 0b00000 }, 
  { 0b00000, 0b00000, 0b00000, 0b00000, 0b01110, 0b01110, 0b01110, 0b01110, 0b01110, 0b01110, 0b01110, 0b01110, 0b01110, 0b01110, 0b01110, 0b00000, 0b00000, 0b00000, 0b00000 }, 
  { 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b01110, 0b01110, 0b01110, 0b01110, 0b01110, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000 }
};*/


const char Seconds_Effect [] = 
   { 0b00001, 0b00010, 0b00100, 0b01000, 0b10000,
     0b10001, 0b10010, 0b10100, 0b11000, 0b11100,
     0b11101, 0b11110, 0b11111, 0b01111, 0b10111,
     0b00111, 0b01011, 0b10011, 0b00011, 0b00101,
     0b01001, 0b10001, 0b00001, 0b00010, 0b00100,
     0b01000, 0b10000, 0b00000, 0b00100, 0b01110,
     0b11111, 0b11011, 0b10001, 0b00000, 0b00001,
     0b00010, 0b00100, 0b01000, 0b10000, 0b01000,
     0b00100, 0b00010, 0b00001, 0b10000, 0b00010,
     0b01000, 0b00100, 0b11011, 0b11101, 0b10111,
     0b11110, 0b01111, 0b10101, 0b01010, 0b10001,
     0b00100, 0b01010, 0b10001, 0b01110, 0b00000
   };
  
/*--------------------------------------------------------------------------------------------------
                                     Character generator

             This table defines the standard ASCII characters in a 5x5 dot format.
             Only 5 LSB per line are used. 
--------------------------------------------------------------------------------------------------*/
const char FontLookup [][5] =
{
    { 0b00000, 0b00000, 0b00000, 0b00000, 0b00000 },  // sp
    { 0b00100, 0b00100, 0b00100, 0b00000, 0b00100 },  // !
    { 0b01010, 0b01010, 0b10100, 0b00000, 0b00000 },  // "
    { 0b01010, 0b11111, 0b01010, 0b11111, 0b01010 },  // #
    { 0b01111, 0b10100, 0b01110, 0b00101, 0b11110 },  // $
    { 0b11001, 0b11010, 0b00100, 0b01011, 0b10011 },  // %
    { 0b01100, 0b10100, 0b01100, 0b10011, 0b01101 },  // &
    { 0b00100, 0b00100, 0b01000, 0b00000, 0b00000 },  // '
    { 0b00100, 0b01000, 0b01000, 0b01000, 0b00100 },  // (
    { 0b01000, 0b00100, 0b00100, 0b00100, 0b01000 },  // )
    { 0b00100, 0b10101, 0b01110, 0b10101, 0b00100 },  // *
    { 0b00000, 0b00100, 0b01110, 0b00100, 0b00000 },  // +
    { 0b00000, 0b00000, 0b00000, 0b00100, 0b01000 },  // ,
    { 0b00000, 0b00000, 0b01110, 0b00000, 0b00000 },  // -
    { 0b00000, 0b00000, 0b00000, 0b00000, 0b00100 },  // .
    { 0b00001, 0b00010, 0b00100, 0b01000, 0b10000 },  // /
    { 0b01100, 0b10010, 0b10110, 0b11010, 0b01100 },  // 0
    { 0b00100, 0b01100, 0b00100, 0b00100, 0b01110 },  // 1
    { 0b11100, 0b00010, 0b01100, 0b10000, 0b11110 },  // 2
    { 0b11100, 0b00010, 0b01100, 0b00010, 0b11100 },  // 3
    { 0b00100, 0b01100, 0b10100, 0b11110, 0b00100 },  // 4
    { 0b11110, 0b10000, 0b01100, 0b00010, 0b11100 },  // 5
    { 0b01100, 0b10000, 0b11100, 0b10010, 0b01100 },  // 6
    { 0b11110, 0b00010, 0b00100, 0b01000, 0b01000 },  // 7
    { 0b01100, 0b10010, 0b01100, 0b10010, 0b01100 },  // 8
    { 0b01100, 0b10010, 0b01100, 0b00010, 0b01100 },  // 9
    { 0b00000, 0b00100, 0b00000, 0b00100, 0b00000 },  // :
    { 0b00000, 0b00100, 0b00000, 0b00100, 0b01000 },  // ;
    { 0b00010, 0b00100, 0b01000, 0b00100, 0b00010 },  // <
    { 0b00000, 0b01110, 0b00000, 0b01110, 0b00000 },  // =
    { 0b01000, 0b00100, 0b00010, 0b00100, 0b01000 },  // >
    { 0b01110, 0b10001, 0b00110, 0b00000, 0b00100 },  // ?
    { 0b01110, 0b10001, 0b10111, 0b10000, 0b01111 },  // @
    { 0b01100, 0b10010, 0b11110, 0b10010, 0b10010 },  // A
    { 0b11100, 0b10010, 0b11100, 0b10010, 0b11100 },  // B
    { 0b01110, 0b10000, 0b10000, 0b10000, 0b01110 },  // C
    { 0b11100, 0b10010, 0b10010, 0b10010, 0b11100 },  // D
    { 0b11110, 0b10000, 0b11100, 0b10000, 0b11110 },  // E
    { 0b11110, 0b10000, 0b11100, 0b10000, 0b10000 },  // F
    { 0b01100, 0b10000, 0b10110, 0b10010, 0b01110 },  // G
    { 0b10010, 0b10010, 0b11110, 0b10010, 0b10010 },  // H
    { 0b01110, 0b00100, 0b00100, 0b00100, 0b01110 },  // I
    { 0b01110, 0b00100, 0b00100, 0b10100, 0b01000 },  // J
    { 0b10010, 0b10100, 0b11000, 0b10100, 0b10010 },  // K
    { 0b10000, 0b10000, 0b10000, 0b10000, 0b11110 },  // L
    { 0b10001, 0b11011, 0b10101, 0b10001, 0b10001 },  // M
    { 0b10010, 0b11010, 0b10110, 0b10010, 0b10010 },  // N
    { 0b01100, 0b10010, 0b10010, 0b10010, 0b01100 },  // O
    { 0b11100, 0b10010, 0b11100, 0b10000, 0b10000 },  // P
    { 0b01100, 0b10010, 0b10010, 0b10100, 0b01010 },  // Q
    { 0b11100, 0b10010, 0b11100, 0b10100, 0b10010 },  // R
    { 0b01110, 0b10000, 0b01100, 0b00010, 0b11100 },  // S
    { 0b11111, 0b00100, 0b00100, 0b00100, 0b00100 },  // T
    { 0b10010, 0b10010, 0b10010, 0b10010, 0b01100 },  // U
    { 0b10001, 0b10001, 0b01010, 0b01010, 0b00100 },  // V
    { 0b10001, 0b10001, 0b10101, 0b11011, 0b10001 },  // W
    { 0b10001, 0b01010, 0b00100, 0b01010, 0b10001 },  // X
    { 0b10001, 0b01010, 0b00100, 0b00100, 0b00100 },  // Y
    { 0b11111, 0b00010, 0b00100, 0b01000, 0b11111 },  // Z
    { 0b01110, 0b01000, 0b01000, 0b01000, 0b01110 },  // [
    { 0b10000, 0b01000, 0b00100, 0b00010, 0b00001 },  // (backslash)
    { 0b01110, 0b00010, 0b00010, 0b00010, 0b01110 },  // ]
    { 0b00100, 0b01010, 0b00000, 0b00000, 0b00000 },  // ^
    { 0b00000, 0b00000, 0b00000, 0b00000, 0b11111 },  // _
    { 0b01000, 0b01000, 0b00100, 0b00000, 0b00000 },  // '
    { 0b01110, 0b00001, 0b01111, 0b10001, 0b01111 },  // a
    { 0b10000, 0b10000, 0b11110, 0b10001, 0b11110 },  // b
    { 0b00000, 0b01110, 0b10000, 0b10000, 0b01110 },  // c
    { 0b00001, 0b00001, 0b01111, 0b10001, 0b01111 },  // d
    { 0b01110, 0b10001, 0b11111, 0b10000, 0b01111 },  // e
    { 0b00110, 0b01001, 0b11100, 0b01000, 0b01000 },  // f
    { 0b01110, 0b10001, 0b01111, 0b00001, 0b11110 },  // g
    { 0b10000, 0b10000, 0b11100, 0b10010, 0b10010 },  // h
    { 0b00100, 0b00000, 0b00100, 0b00100, 0b00100 },  // i
    { 0b00100, 0b00000, 0b00100, 0b00100, 0b01000 },  // j
    { 0b10010, 0b10100, 0b11000, 0b10100, 0b10010 },  // k
    { 0b01100, 0b00100, 0b00100, 0b00100, 0b01110 },  // l
    { 0b01010, 0b10101, 0b10101, 0b10001, 0b10001 },  // m
    { 0b11110, 0b10001, 0b10001, 0b10001, 0b10001 },  // n
    { 0b01100, 0b10010, 0b10010, 0b10010, 0b01100 },  // o
    { 0b11100, 0b10010, 0b11100, 0b10000, 0b10000 },  // p
    { 0b01110, 0b10010, 0b01110, 0b00010, 0b00011 },  // q
    { 0b10110, 0b11001, 0b10000, 0b10000, 0b10000 },  // r
    { 0b01110, 0b10000, 0b01100, 0b00010, 0b11100 },  // s
    { 0b01000, 0b11110, 0b01000, 0b01000, 0b00110 },  // t
    { 0b10010, 0b10010, 0b10010, 0b10010, 0b01100 },  // u
    { 0b10001, 0b10001, 0b01010, 0b01010, 0b00100 },  // v
    { 0b10001, 0b10001, 0b10101, 0b10101, 0b01010 },  // w
    { 0b10001, 0b01010, 0b00100, 0b01010, 0b10001 },  // x
    { 0b10010, 0b10010, 0b01110, 0b00010, 0b11100 },  // y
    { 0b00000, 0b11110, 0b00100, 0b01000, 0b11110 },  // z
    { 0b01010, 0b11111, 0b11111, 0b01110, 0b00100 },  // {  [heart]
    { 0b00100, 0b10101, 0b00100, 0b01010, 0b10001 },  // |  [stick figure man]
    { 0b00100, 0b01000, 0b11111, 0b01000, 0b00100 },  // }  [arrow]
    //{ 0b10101, 0b01010, 0b10101, 0b01010, 0b10101 }   // ~  [end of message/checkered flag]  
    { 0b11111, 0b11111, 0b11111, 0b11111, 0b11111 },  
    { 0b10010, 0b10101, 0b10101, 0b10101, 0b10010 },  // 10
    { 0b10010, 0b10010, 0b10010, 0b10010, 0b10010 },  // 11
    { 0b10110, 0b10001, 0b10010, 0b10100, 0b10111 },  // 12
};
      
/* unused character elements
 *
 * { 0b00100, 0b00100, 0b00100, 0b00100, 0b00100 }  |
 * { 0b01001, 0b10101, 0b10010, 0b00000, 0b00000 }  ~
 * { 0b01010, 0b10101, 0b10001, 0b01010, 0b00100 }  [heart outline]
 * { 0b00001, 0b01001, 0b11111, 0b01000, 0b00000 }  [return arrow]
 * { 0b00110, 0b01000, 0b11100, 0b01001, 0b11111 }  [uk pound]
 * { 0b00100, 0b11111, 0b00100, 0b00100, 0b00100 }  [cross]       
 *
 */
     
// taken from http://www.fileformat.info/mirror/egff/ch06_04.htm
#define INVERTBITS(b)   (~(b))
#define REVERSEBITS(b)  (BitReverseTable[b])

const unsigned char BitReverseTable[256] =
{
0x00, 0x80, 0x40, 0xc0, 0x20, 0xa0, 0x60, 0xe0,
0x10, 0x90, 0x50, 0xd0, 0x30, 0xb0, 0x70, 0xf0,
0x08, 0x88, 0x48, 0xc8, 0x28, 0xa8, 0x68, 0xe8,
0x18, 0x98, 0x58, 0xd8, 0x38, 0xb8, 0x78, 0xf8,
0x04, 0x84, 0x44, 0xc4, 0x24, 0xa4, 0x64, 0xe4,
0x14, 0x94, 0x54, 0xd4, 0x34, 0xb4, 0x74, 0xf4,
0x0c, 0x8c, 0x4c, 0xcc, 0x2c, 0xac, 0x6c, 0xec,
0x1c, 0x9c, 0x5c, 0xdc, 0x3c, 0xbc, 0x7c, 0xfc,
0x02, 0x82, 0x42, 0xc2, 0x22, 0xa2, 0x62, 0xe2,
0x12, 0x92, 0x52, 0xd2, 0x32, 0xb2, 0x72, 0xf2,
0x0a, 0x8a, 0x4a, 0xca, 0x2a, 0xaa, 0x6a, 0xea,
0x1a, 0x9a, 0x5a, 0xda, 0x3a, 0xba, 0x7a, 0xfa,
0x06, 0x86, 0x46, 0xc6, 0x26, 0xa6, 0x66, 0xe6,
0x16, 0x96, 0x56, 0xd6, 0x36, 0xb6, 0x76, 0xf6,
0x0e, 0x8e, 0x4e, 0xce, 0x2e, 0xae, 0x6e, 0xee,
0x1e, 0x9e, 0x5e, 0xde, 0x3e, 0xbe, 0x7e, 0xfe,
0x01, 0x81, 0x41, 0xc1, 0x21, 0xa1, 0x61, 0xe1,
0x11, 0x91, 0x51, 0xd1, 0x31, 0xb1, 0x71, 0xf1,
0x09, 0x89, 0x49, 0xc9, 0x29, 0xa9, 0x69, 0xe9,
0x19, 0x99, 0x59, 0xd9, 0x39, 0xb9, 0x79, 0xf9,
0x05, 0x85, 0x45, 0xc5, 0x25, 0xa5, 0x65, 0xe5,
0x15, 0x95, 0x55, 0xd5, 0x35, 0xb5, 0x75, 0xf5,
0x0d, 0x8d, 0x4d, 0xcd, 0x2d, 0xad, 0x6d, 0xed,
0x1d, 0x9d, 0x5d, 0xdd, 0x3d, 0xbd, 0x7d, 0xfd,
0x03, 0x83, 0x43, 0xc3, 0x23, 0xa3, 0x63, 0xe3,
0x13, 0x93, 0x53, 0xd3, 0x33, 0xb3, 0x73, 0xf3,
0x0b, 0x8b, 0x4b, 0xcb, 0x2b, 0xab, 0x6b, 0xeb,
0x1b, 0x9b, 0x5b, 0xdb, 0x3b, 0xbb, 0x7b, 0xfb,
0x07, 0x87, 0x47, 0xc7, 0x27, 0xa7, 0x67, 0xe7,
0x17, 0x97, 0x57, 0xd7, 0x37, 0xb7, 0x77, 0xf7,
0x0f, 0x8f, 0x4f, 0xcf, 0x2f, 0xaf, 0x6f, 0xef,
0x1f, 0x9f, 0x5f, 0xdf, 0x3f, 0xbf, 0x7f, 0xff
};

const unsigned char RiderTable[][8][12] = {
{      // 8-4-2-1
{ 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111 }, 
{ 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b11111, 0b11111, 0b11111 }, 
{ 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111 }, 
{ 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b11111, 0b11111, 0b11111 }, 
{ 0b01010, 0b01010, 0b01010, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111 }, 
{ 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b11111, 0b11111, 0b11111 }, 
{ 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111 }, 
{ 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b11111, 0b11111, 0b11111 }, 
},

{      // 1-2-4-8
{ 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111 }, 
{ 0b11111, 0b11111, 0b11111, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000 }, 
{ 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000 }, 
{ 0b11111, 0b11111, 0b11111, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000 }, 
{ 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b01010, 0b01010, 0b01010 }, 
{ 0b11111, 0b11111, 0b11111, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000 }, 
{ 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000 }, 
{ 0b11111, 0b11111, 0b11111, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000 }, 
},
 
{       // 0-8-1-2
{ 0b00000, 0b00000, 0b00000, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111 }, 
{ 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b11111, 0b11111, 0b11111, 0b00000, 0b00000, 0b00000 }, 
{ 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111 }, 
{ 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b11111, 0b11111, 0b11111, 0b00000, 0b00000, 0b00000 }, 
{ 0b00000, 0b00000, 0b00000, 0b01010, 0b01010, 0b01010, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111 }, 
{ 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b11111, 0b11111, 0b11111, 0b00000, 0b00000, 0b00000 }, 
{ 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111 }, 
{ 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b11111, 0b11111, 0b11111, 0b00000, 0b00000, 0b00000 }, 
},

{      // 0-1-2-4
{ 0b00000, 0b00000, 0b00000, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111 }, 
{ 0b00000, 0b00000, 0b00000, 0b11111, 0b11111, 0b11111, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000 }, 
{ 0b00000, 0b00000, 0b00000, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b00000, 0b00000, 0b00000 }, 
{ 0b00000, 0b00000, 0b00000, 0b11111, 0b11111, 0b11111, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000 }, 
{ 0b00000, 0b00000, 0b00000, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111 }, 
{ 0b00000, 0b00000, 0b00000, 0b11111, 0b11111, 0b11111, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000 }, 
{ 0b00000, 0b00000, 0b00000, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b00000, 0b00000, 0b00000 }, 
{ 0b00000, 0b00000, 0b00000, 0b11111, 0b11111, 0b11111, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000 }, 
},

{      // 2-1-8-0
{ 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b00000, 0b00000, 0b00000 }, 
{ 0b00000, 0b00000, 0b00000, 0b11111, 0b11111, 0b11111, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000 }, 
{ 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000 }, 
{ 0b00000, 0b00000, 0b00000, 0b11111, 0b11111, 0b11111, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000 }, 
{ 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b01010, 0b01010, 0b01010, 0b00000, 0b00000, 0b00000 }, 
{ 0b00000, 0b00000, 0b00000, 0b11111, 0b11111, 0b11111, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000 }, 
{ 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000 }, 
{ 0b00000, 0b00000, 0b00000, 0b11111, 0b11111, 0b11111, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000 }, 
},
 
{      // 4-2-1-0
{ 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b00000, 0b00000, 0b00000 }, 
{ 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b11111, 0b11111, 0b11111, 0b00000, 0b00000, 0b00000 }, 
{ 0b00000, 0b00000, 0b00000, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b00000, 0b00000, 0b00000 }, 
{ 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b11111, 0b11111, 0b11111, 0b00000, 0b00000, 0b00000 }, 
{ 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b00000, 0b00000, 0b00000 }, 
{ 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b11111, 0b11111, 0b11111, 0b00000, 0b00000, 0b00000 }, 
{ 0b00000, 0b00000, 0b00000, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b11111, 0b00000, 0b00000, 0b00000 }, 
{ 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b00000, 0b11111, 0b11111, 0b11111, 0b00000, 0b00000, 0b00000 }, 
},
};

const char RiderStates[] =   { 0, 0, 0, 2, 3, 1, 1, 1, 4, 5 };
const char RiderPosition[] = { 0, 3, 6, 6, 6, 6, 3, 0, 0, 0 };

  
 

