// Modified source, based on Kingpin's original code.
// Modification by sunshine@pegasus.ch
//   DefCon 15, 2007
// Feel free to modify, hack, distribute or whatever you want to do with it.
// Honorable mentions of the source (and it's origins is appreciated (but not required)

#include <hidef.h> /* for EnableInterrupts macro */
#include <stdio.h>
#include <stdlib.h>
#include "derivative.h" /* include peripheral declarations */
#include "demo9S08QG8.h" /*include demo board declarations */
#include "serial.h"

   
state_type state = SLEEP; // Start up in this mode
unsigned char ScrollSpeed = 3; // scroll speed (1 = slowest, 5 = fastest)
unsigned char MsgChar; // currently selected character for user-customizable scrolling text message

unsigned char scroll_flag = 0; // if set, scroll the display, else remain static
unsigned char state_change_flag = 1; // if set, we need to change state/mode
 
unsigned char fb[MSG_LENGTH]; // frame buffer
unsigned char shiftreg[NUMBER_OF_74595];
unsigned char row_count = 0; // current row of message buffer that we'll copy to fb (for scrolling)
unsigned char ScrollDirection = 0;  // 0 = forward, 1 = back
unsigned char Effect = 0; // bit 1 = forward/backward bounce, bit 2 = blink 
unsigned char off_counter = 0;  // timer for blinking
unsigned char clock_hours = 0;
unsigned char clock_minutes = 0;
unsigned char clock_seconds = 0;
unsigned char clock_count = 0;
unsigned char clock_refresh = 0;  // forceing clock display update if 1
unsigned char clock_stop = 0;
unsigned char rider_step = 0;
 
void main(void) 
{
  signed char i;
  unsigned char str_tmp[CHAR_LENGTH + 1]; // buffer for user-generated text message + null
  
  disable_scrolling();
  MCU_init(); // call Device Initialization from MCUInit.c
  
  memset(fb, 0x1F, NUM_ROWS);  // turn on all LEDs in the entire matrix (for testing)
  delay_ms(250);
  
  clear_matrix();     
  memset(str_tmp, 0x00, CHAR_LENGTH + 1); 

  //(void)printf ("\n\n\rLet's get this party started!\n\n\r");
  
  // sorry for this spaghetti-like state machine - i'm just a lowly electrical engineer :)
  while(1)
  {  
    if (state_change_flag) // if there has been a change in state
  	{
  	  state_change_flag = 0;
MAINLOOP:  	  	  
  		switch (state) // ...then enter the new state
    	{
    	  case SCROLL:
    	    if (str_tmp[0] == '\0') // if no user-programmed message exists...
    	      LedStr((char*)str_default); // ...scroll the default programmed text message
    	    else // otherwise display the user-programmed message
    	    {
    	      if (strcmp(str_tmp, MAGIC_PASSWORD) == 0)
    	        LedStr((char*)str_easteregg); // ssshhh!
    	      else
    	        LedStr(str_tmp);
    	    }
    	      	    
    	    // set scroll speed based on user selection
    	    switch (ScrollSpeed)
    	    {
    	      case 1:
    	        MTIMMOD = 24; // 205ms
    	        break;
    	      case 2:
    	        MTIMMOD = 17; // 147ms
    	        break;
    	      case 4:
    	        MTIMMOD = 6; // 57ms
    	        break;
    	      case 5:
    	        MTIMMOD = 2; // 24ms
    	        break;
    	      case 3:
    	      default:
    	        MTIMMOD = 10; // 90ms
    	        break;
    	    }
    	    
    	    enable_scrolling();    	    
MOREWAIT:
    	    while (SW1 || SW2) asm ("wait"); // wait until no more buttons are pressed
          while (!SW1) asm ("wait");       // just hang out here until it's time to change modes
    	    while (SW1)         // wait until button is released
    	    {
    	      asm ("wait");
    	      if (SW1 && SW2) // if both buttons pressed, don't do anything
    	        goto MOREWAIT;
    	    }
    	    MTIMMOD = 10; // set scroll speed back to default (90ms)
    	    change_state();
    	    break;

    	  case CLOCK:
     	    disable_scrolling();    	    
MOREWAIT_CLOCK:
    	    while (SW1 || SW2) asm ("wait"); // wait until no more buttons are pressed
          while (!SW1) asm ("wait");       // just hang out here until it's time to change modes
    	    while (SW1)         // wait until button is released
    	    {
    	      asm ("wait");
    	      if (SW1 && SW2) // if both buttons pressed, don't do anything
    	        goto MOREWAIT_CLOCK;
    	    }
    	    change_state();
    	    break;

    	  case RIDER:
     	    disable_scrolling();    	    
MOREWAIT_RIDER:
    	    while (SW1 || SW2) asm ("wait"); // wait until no more buttons are pressed
          while (!SW1) asm ("wait");       // just hang out here until it's time to change modes
    	    while (SW1)         // wait until button is released
    	    {
    	      asm ("wait");
    	      if (SW1 && SW2) // if both buttons pressed, don't do anything
    	        goto MOREWAIT_RIDER;
    	    }
    	    change_state();
    	    break;

    	  case SET_MSG:
    	    LedStr("TEXT");
     	    enable_scrolling();
MOREWAIT_MSG:
    	    while (SW1 || SW2) asm ("wait");   // wait until no more buttons are pressed
     	    while (!SW1 && !SW2) asm ("wait"); // wait until a button is pressed
     	    if (SW1)
     	    {
     	      while (SW1) // wait until button is released
     	      { 
     	        asm ("wait");
     	        if (SW1 && SW2) // if both buttons are pressed, don't do anything
     	          goto MOREWAIT_MSG;
     	      }
     	      change_state();
            break;
     	    }
     	    else if (SW2)
     	    {
     	      while (SW2) // wait until button is released
     	      {
     	        asm ("wait");
     	        if (SW1 && SW2) // if both buttons are pressed, don't do anything
     	          goto MOREWAIT_MSG;
     	      }
     	      disable_scrolling();
     	      i = 0; // pointer into array
     	      MsgChar = 'A'; // default character
     	      while (1)
     	      {
     	        asm ("wait")
     	        if (SW1)
     	        {
     	          while (SW1) // wait until button is released
     	          { 
     	            asm ("wait");
     	            if (SW1 && SW2) // if both buttons pressed...
     	            {
     	              while (SW1 || SW2) asm ("wait");   // wait until no more buttons are pressed
     	              switch (MsgChar)
     	              {
     	                case '}': // arrow selected, erase previous character 
     	                  i--;
     	                  if (i < 0) i = 0;     	                  
     	                  goto DISPTEXT;
     	                  break;
     	                case '~': // checkered flag selected, end mode and return to scrolling message 
     	                  str_tmp[i] = '\0'; // remove end-of-message character     	                  
     	                  state = SCROLL;
     	                  goto MAINLOOP; // get out of this mess of nested loops     	                
     	                  break;
     	                default: // add selected character to scrolling message array/fb
     	                  if (i < CHAR_LENGTH) // if we haven't exceeded the maximum message size...
     	                    i++; 
     	                  else // force end-of-message character to notify user that they can't enter any more text
     	       	            MsgChar = '~'; 
                        goto DISPTEXT;
                        break;
     	              }
     	            }
     	          }
                MsgChar++; // increment character
                if (MsgChar > 0x81) MsgChar = 0x20; 
     	        }
     	        else if (SW2)
     	        {
     	          while (SW2) // wait until button is released
     	          {
     	            asm ("wait");
     	            if (SW1 && SW2) // if both buttons pressed...
     	            {
     	              while (SW1 || SW2) asm ("wait");   // wait until no more buttons are pressed
     	              switch (MsgChar)
     	              {
     	                case '}': // arrow selected, erase previous character 
     	                  i--;
     	                  if (i < 0) i = 0;
     	                  goto DISPTEXT;
     	                  break;
     	                case '~': // checkered flag selected, end mode and return to scrolling message 
     	                  str_tmp[i] = '\0'; // remove end-of-message character
     	                  state = SCROLL;
     	                  goto MAINLOOP; // get out of this mess of nested loops     	                
     	                  break;
     	                default: // add selected character to scrolling message array/fb
     	                  if (i < CHAR_LENGTH) // if we haven't exceeded the maximum message size...
     	                    i++;
     	                  else // force end-of-message character to notify user that they can't enter any more text
     	                    MsgChar = '~'; 
     	                  goto DISPTEXT;                              	                  
                        break;
     	              }
     	            }
     	          }
                MsgChar--; // increment character
                if (MsgChar < 0x20) MsgChar = 0x81;
     	        }
DISPTEXT:
     	        str_tmp[i] = MsgChar; // append currently selected character to array
     	        str_tmp[i + 1] = '\0';
     	        row_count = 6 + (i * 6); // skip the blank spaces in fb
              LedStr(str_tmp); 
     	      }
     	      break;
     	    }
     	    break;
    	  case SET_CLOCK:
    	    LedStr("CLOCK SET");
     	    enable_scrolling();
MOREWAIT_CLOCKSET:
    	    while (SW1 || SW2) asm ("wait");   // wait until no more buttons are pressed
     	    while (!SW1 && !SW2) asm ("wait"); // wait until a button is pressed
     	    if (SW1)
     	    {
     	      while (SW1) // wait until button is released
     	      {
     	        asm ("wait");
     	        if (SW1 && SW2) // if both buttons are pressed, don't do anything
     	          goto MOREWAIT_CLOCKSET;
     	      }
     	      change_state();
     	      break;
     	    }
     	    else if (SW2)
     	    {
     	      while (SW2) // wait until button is released
     	      {
     	        asm ("wait");
     	        if (SW1 && SW2) // if both buttons are pressed, don't do anything
     	          goto MOREWAIT_CLOCKSET;
     	      }
     	      disable_scrolling();
     	      clock_stop = 1;
     	      clock_seconds = 53; // use nice : pattern for sec 53
     	      while (1)
     	      {
       	      LedClock();
     	        asm ("wait");
     	        if (SW1)
     	        {
     	          while (SW1) // wait until button is released
     	          {
     	            asm ("wait");
     	            if (SW1 && SW2) // if both buttons pressed, return to scrolling message
     	            {
     	              clock_seconds = 0;
     	              clock_stop = 0;
     	              state = CLOCK;
     	              goto MAINLOOP; // get out of this mess of nested loops
     	            }
     	          }
     	          clock_hours++;
                if (clock_hours > 11) clock_hours = 0;     	          
     	        }
     	        else if (SW2)
     	        {
     	          while (SW2) // wait until button is released
     	          {
     	            asm ("wait");
     	            if (SW1 && SW2) // if both buttons pressed, return to scrolling message
     	            {
     	              clock_seconds = 0;
     	              clock_stop = 0;
     	              state = CLOCK;
                    goto MAINLOOP; // get out of this mess of nested loops
     	            }
     	          }
     	          clock_minutes++;
     	          if (clock_minutes > 59) clock_minutes = 0;
     	        }
     	      }
     	      break;
     	    }
     	    break;
    	  case SET_SPEED:
    	    LedStr("SPEED");
     	    enable_scrolling();
MOREWAIT_SPEED:
    	    while (SW1 || SW2) asm ("wait");   // wait until no more buttons are pressed
     	    while (!SW1 && !SW2) asm ("wait"); // wait until a button is pressed
     	    if (SW1)
     	    {
     	      while (SW1) // wait until button is released
     	      {
     	        asm ("wait");
     	        if (SW1 && SW2) // if both buttons are pressed, don't do anything
     	          goto MOREWAIT_SPEED;
     	      }
     	      change_state();
     	      break;
     	    }
     	    else if (SW2)
     	    {
     	      while (SW2) // wait until button is released
     	      {
     	        asm ("wait");
     	        if (SW1 && SW2) // if both buttons are pressed, don't do anything
     	          goto MOREWAIT_SPEED;
     	      }
     	      disable_scrolling();
     	      while (1)
     	      {
     	        asm ("wait");
     	        if (SW1)
     	        {
     	          while (SW1) // wait until button is released
     	          {
     	            asm ("wait");
     	            if (SW1 && SW2) // if both buttons pressed, return to scrolling message
     	            {
     	              state = SCROLL;
     	              goto MAINLOOP; // get out of this mess of nested loops
     	            }
     	          }
     	          ScrollSpeed++;
                if (ScrollSpeed > 5) ScrollSpeed = 1;     	          
     	        }
     	        else if (SW2)
     	        {
     	          while (SW2) // wait until button is released
     	          {
     	            asm ("wait");
     	            if (SW1 && SW2) // if both buttons pressed, return to scrolling message
     	            {
     	              state = SCROLL;
                    goto MAINLOOP; // get out of this mess of nested loops
     	            }
     	          }
     	          ScrollSpeed--;
     	          if (ScrollSpeed < 1) ScrollSpeed = 5;
     	        }
     	        row_count = 6; // skip the blank spaces in fb
              LedStr(num2asc(ScrollSpeed) + 1); // in serial.c. ignore the first byte, which will be 0 for a single-digit input
     	      }
     	      break;
     	    }
     	    break;
    	  case SET_BOUNCE:
    	    LedStr("BOUNCE");
     	    enable_scrolling();
MOREWAIT_BOUNCE:
    	    while (SW1 || SW2) asm ("wait");   // wait until no more buttons are pressed
     	    while (!SW1 && !SW2) asm ("wait"); // wait until a button is pressed
     	    if (SW1)
     	    {
     	      while (SW1) // wait until button is released
     	      {
     	        asm ("wait");
     	        if (SW1 && SW2) // if both buttons are pressed, don't do anything
     	          goto MOREWAIT_BOUNCE;
     	      }
     	      change_state();
     	      break;
     	    }
     	    else if (SW2)
     	    {
     	      while (SW2) // wait until button is released
     	      {
     	        asm ("wait");
     	        if (SW1 && SW2) // if both buttons are pressed, don't do anything
     	          goto MOREWAIT_BOUNCE;
     	      }
     	      disable_scrolling();
     	      while (1)
     	      {
     	        asm ("wait");
     	        if (SW1)
     	        {
     	          while (SW1) // wait until button is released
     	          {
     	            asm ("wait");
     	            if (SW1 && SW2) // if both buttons pressed, return to scrolling message
     	            {
     	              state = SCROLL;
     	              goto MAINLOOP; // get out of this mess of nested loops
     	            }
     	          }
     	          Effect = Effect ^ 0x1;     	          
     	        }
     	        else if (SW2)
     	        {
     	          while (SW2) // wait until button is released
     	          {
     	            asm ("wait");
     	            if (SW1 && SW2) // if both buttons pressed, return to scrolling message
     	            {
     	              state = SCROLL;
                    goto MAINLOOP; // get out of this mess of nested loops
     	            }
     	          }
     	          Effect = Effect ^ 0x1;
     	        }
     	        row_count = 6; // skip the blank spaces in fb
     	        switch (Effect & 0x1) 
     	        {
     	          case 0:
     	            LedStr("N");
     	            break;
     	          case 1:
     	          default:
     	            LedStr("Y");
     	            break;
     	        }
     	      }
     	      break;
     	    }
     	    break;
     	  case SET_BLINK:
    	    LedStr("BLINK");
     	    enable_scrolling();
MOREWAIT_BLINK:
    	    while (SW1 || SW2) asm ("wait");   // wait until no more buttons are pressed
     	    while (!SW1 && !SW2) asm ("wait"); // wait until a button is pressed
     	    if (SW1)
     	    {
     	      while (SW1) // wait until button is released
     	      {
     	        asm ("wait");
     	        if (SW1 && SW2) // if both buttons are pressed, don't do anything
     	          goto MOREWAIT_BLINK;
     	      }
     	      change_state();
     	      break;
     	    }
     	    else if (SW2)
     	    {
     	      while (SW2) // wait until button is released
     	      {
     	        asm ("wait");
     	        if (SW1 && SW2) // if both buttons are pressed, don't do anything
     	          goto MOREWAIT_BLINK;
     	      }
     	      disable_scrolling();
     	      while (1)
     	      {
     	        asm ("wait");
     	        if (SW1)
     	        {
     	          while (SW1) // wait until button is released
     	          {
     	            asm ("wait");
     	            if (SW1 && SW2) // if both buttons pressed, return to scrolling message
     	            {
     	              state = SCROLL;
     	              goto MAINLOOP; // get out of this mess of nested loops
     	            }
     	          }
     	          Effect = Effect ^ 0x2;    	          
     	        }
     	        else if (SW2)
     	        {
     	          while (SW2) // wait until button is released
     	          {
     	            asm ("wait");
     	            if (SW1 && SW2) // if both buttons pressed, return to scrolling message
     	            {
     	              state = SCROLL;
                    goto MAINLOOP; // get out of this mess of nested loops
     	            }
     	          }
     	          Effect = Effect ^ 0x2;
     	        }
     	        row_count = 6; // skip the blank spaces in fb
     	        switch (Effect & 0x2) 
     	        {
     	          case 0:
     	            LedStr("N");
     	            break;
     	          case 2:
     	          default:
     	            LedStr("Y");
     	            break;
     	        }
     	      }
     	      break;
     	    }
     	    break;
    	  case POV:
          LedStr("POV"); 
     	    enable_scrolling();
MOREWAIT_POV:
    	    while (SW1 || SW2) asm ("wait");   // wait until no more buttons are pressed
     	    while (!SW1 && !SW2) asm ("wait"); // wait until a button is pressed
     	    if (SW1)
     	    {
     	      while (SW1) // wait until button is released
     	      {
     	        asm ("wait");
     	        if (SW1 && SW2) // if both buttons are pressed, don't do anything
     	          goto MOREWAIT_POV;
     	      }     	      
     	      change_state();
     	      break;
     	    }
     	    else if (SW2)
     	    {
     	      while (SW2) // wait until button is released
     	      {
     	        asm ("wait");
     	        if (SW1 && SW2) // if both buttons are pressed, don't do anything
     	          goto MOREWAIT_POV;
     	      }     	      
     	      disable_scrolling();
     	      SRTISC_RTIE = 0; // disable RTI - too slow for our purposes
     	      while (!SW1 || !SW2) // do this state until both buttons are pressed
     	      {
     	        // remember that you have to spin the badge in the proper direction in order to display the correct, non-mirrored image
              // the badge has no sense of where it is located, so it just displays the graphics over and over again until it's time to change modes
              // you could do some cool tweaks with the accelerometer here, like reverse the drawing direction when the badge is going in the other direction
              for (i = 0; i < POV_MAX_FRAME; ++i) 
      	      {
      	        memcpy(fb, &POV_buffer[i], NUM_ROWS);
      	        draw_buffer(fb);      	     
      	        //delay(1); // frame rate speed - determines how fast the image is "drawn" into the air
      	      }
     	      }
     	      SRTISC_RTIE = 1; // re-enable RTI
     	      change_state();
            break;
     	    }
    	    break;
    	  case SLEEP:
    	  default:
    	    disable_scrolling();
    	    clear_matrix();
    	    while (SW1 || SW2) asm ("wait"); // wait until no more buttons are pressed
//  Don't disable interupts - need it for clock
//    	    SRTISC_RTIE = 0;    // disable RTI, since that could wake us up
BACKTOSLEEP:
          asm ("wait");  // go to sleep (we're configured for STOP3 standby mode in Device Initialization) until keypress (only STOP3 wakes on KBI)         

          // we will wake on a KBI from SW1 or SW2, but we're only interested in SW1
          if (SW1)
          {
            delay_ms(25);  // poor man's debounce added to prevent spurious interrupts from QT100 touch sensors
            if (SW1)
            {
              while (SW1)
              {
                asm ("wait");
                if (SW1 && SW2) // if both buttons are pressed, don't do anything
                  goto BACKTOSLEEP;
              }
              SRTISC_RTIE = 1;    // re-enable RTI
    	        change_state();     // when we're awakened, time to change state              
              break;
            }
            else
              goto BACKTOSLEEP; 
          }
          else
            goto BACKTOSLEEP;
    	}
  	}
  } /* loop forever */
  /* please make sure that you never leave main */
}


void clear_matrix(void)
{
  memset(fb, 0x00, MSG_LENGTH); 
}


// this function will draw the entire matrix (all 19 rows)
// it needs to be refreshed as often as possible, since we're using multiplexing
void draw_buffer(unsigned char* buf) // pass pointer to frame buffer
{
  unsigned char i;
  
  // the first 3 rows are a special case since we have to shift in row control along with column data
  shiftreg[U4] = REVERSEBITS(buf[0]) | 0b011;  
  shiftreg[U5] = 0xFF;
  shiftreg[U6] = 0xFF;    
  write_expanded_outputs(shiftreg);  
   
  shiftreg[U4] = REVERSEBITS(buf[1]) | 0b101;
  shiftreg[U5] = 0xFF;
  shiftreg[U6] = 0xFF;    
  write_expanded_outputs(shiftreg);  
  
  shiftreg[U4] = REVERSEBITS(buf[2]) | 0b110;
  shiftreg[U5] = 0xFF;
  shiftreg[U6] = 0xFF;    
  write_expanded_outputs(shiftreg);  

  for (i = 3; i <= 10; ++i)
  {
    shiftreg[U4] = REVERSEBITS(buf[i]) | 0b111;
    shiftreg[U5] = (0x80 >> (unsigned char)(i-3)) ^ 0xFF; // select the row to enable
    shiftreg[U6] = 0xFF;    
    write_expanded_outputs(shiftreg);  
  }

  for (i = 11; i <= 18; ++i)
  {
    shiftreg[U4] = REVERSEBITS(buf[i]) | 0b111;
    shiftreg[U5] = 0xFF;
    shiftreg[U6] = (0x80 >> (unsigned char)(i-11)) ^ 0xFF;   
    write_expanded_outputs(shiftreg);  
  }
  
  shiftreg[U4] = 0xFF;
  shiftreg[U5] = 0xFF;
  shiftreg[U6] = 0xFF;    
  write_expanded_outputs(shiftreg);      
}  


// this function will draw the entire matrix (all 19 rows)
// it needs to be refreshed as often as possible, since we're using multiplexing
void draw_rider(unsigned char start, const unsigned char* buf) // pass pointer to frame buffer
{
  unsigned char i, j, pixel;
  
  i=0;
  pixel = start + 12;
  // the first 3 rows are a special case since we have to shift in row control along with column data
  if ((pixel > 0) && (pixel <= 12))
  {
    shiftreg[U4] = REVERSEBITS(buf[i++]) | 0b011;  
    shiftreg[U5] = 0xFF;
    shiftreg[U6] = 0xFF;    
    write_expanded_outputs(shiftreg);  
  }
  pixel--;
  
  if ((pixel > 0) && (pixel <= 12))
  {
    shiftreg[U4] = REVERSEBITS(buf[i++]) | 0b101;
    shiftreg[U5] = 0xFF;
    shiftreg[U6] = 0xFF;    
    write_expanded_outputs(shiftreg);  
  }
  pixel--;
  
  
  if ((pixel > 0) && (pixel <= 12))
  {
    shiftreg[U4] = REVERSEBITS(buf[i++]) | 0b110;
    shiftreg[U5] = 0xFF;
    shiftreg[U6] = 0xFF;    
    write_expanded_outputs(shiftreg);  
  }
  pixel--;
  
  for (j = 0; j <= 7; ++j)
  {
    if ((pixel > 0) && (pixel <= 12)) 
    {
       shiftreg[U4] = REVERSEBITS(buf[i++]) | 0b111;
       shiftreg[U5] = (0x80 >> (unsigned char)(j)) ^ 0xFF; // select the row to enable
       shiftreg[U6] = 0xFF;
       write_expanded_outputs(shiftreg);  
    } 
    pixel--;
  }

  for (j = 0; j <= 6; ++j)
  {
    if ((pixel > 0) && (pixel <= 12)) 
    {
       shiftreg[U4] = REVERSEBITS(buf[i++]) | 0b111;
       shiftreg[U5] = 0xFF;
       shiftreg[U6] = (0x80 >> (unsigned char)(j)) ^ 0xFF;   
       write_expanded_outputs(shiftreg);  
    } 
    pixel--;
  }
  
  shiftreg[U4] = 0xFF;
  shiftreg[U5] = 0xFF;
  shiftreg[U6] = 0xFF;    
  write_expanded_outputs(shiftreg);      
}  


/*--------------------------------------------------------------------------------------------------

  Name         :  LedStr
  Description  :  Loads a string into the frame buffer
  Argument(s)  :  dataPtr -> Pointer to null terminated ASCII string to display.
  Return value :  None.
  
  NOTE : Be sure the passed string is not longer than the frame buffer

--------------------------------------------------------------------------------------------------*/

void LedStr (char *dataPtr)
{
  unsigned char i, j;

  // we waste a lot of RAM here with 40 bytes of blanks...
  for (i = 0; i < 20; i++) // add enough space as header to start text from bottom of matrix
       fb[i] = 0x00; 
    
  while ( *dataPtr )
  {
    if ( (*dataPtr < 0x20) || (*dataPtr > 0x81) ) // If non-printable character,
        *dataPtr = 32;  //  Convert to a blank.
    
    for (j = 0; j < 5; j++)
    {
         fb[i] = FontLookup[*dataPtr - 32][j];
         i++;
    }
    
    if (*dataPtr != 32)  // if we're not printing a space
    {
      fb[i] = 0x00; // add a single blank line in between characters
      i++;
    }
    else
      i--; // remove a blank line to keep words closer together
    
    dataPtr++;
  }
  
  for (j = 0; j < 20; j++, i++) // add enough space to completely scroll through the matrix
       fb[i] = 0x00; 
     
  fb[i] = 0x80; // add end-of-message flag
}


void LedClock (void) 
{
unsigned char i, digit;

  // Seconds Display
  fb[6] = Seconds_Effect[clock_seconds];

  // Update Minutes Display
  digit = (clock_minutes % 10) + 16;
  for (i = 0; i < 5; i++) 
  {
     fb[i+14] = FontLookup[digit][i];
  }
  digit = (clock_minutes / 10) + 16;
  for (i = 0; i < 5; i++) 
  {
     fb[i+8] = FontLookup[digit][i];
  }
          
  // Update Hour Display
  if (clock_hours == 0) 
     digit = 97;
  else if (clock_hours > 9) 
     digit = 85 + clock_hours;
  else 
     digit = clock_hours + 16;
  for (i = 0; i < 5; i++) 
  {
     fb[i] = FontLookup[digit][i];
  }
}


// interrupt service routines

void KBI_ISR (void) 
{ 
  KBISC_KBACK = 1;  // Clear pending keyboard interrupts
}


void RTI_ISR (void) // 8ms
{
  
  // set corner LEDs as indicators if buttons are pressed (only if not scrolling)
  /*if (!scroll_flag)
  {
    if (SW1)
     fb[row_count] |= 0x10; // turn on upper left LED   
    else
     fb[row_count] &= 0x0F; // turn it off
    
    if (SW2)
     fb[row_count] |= 0x01; // turn on upper right LED
    else
     fb[row_count] &= 0x17; // turn it off
  }*/
  
  // Update clock
  clock_count++;
  if ((clock_count > 117) && (clock_stop == 0))
  {
     clock_count = 0;
     clock_seconds++;
     if (clock_seconds > 59) 
     {
        clock_seconds = 0;
        clock_minutes++;
        if (clock_minutes > 59) 
        {
           clock_minutes = 0;
           clock_hours++;
           if (clock_hours > 11)  
              clock_hours = 0;
        }
     }
     // Clock display update if we are in clock mode  
     if (state == CLOCK)
     {
        LedClock();
     }
  }
          
  off_counter++;
  if (off_counter > 80)
     off_counter = 0;
  if ((Effect & 0x2) && (state == SCROLL))  // Blinking
  {
     if (off_counter > 40)
        draw_buffer(fb + row_count);
  } else 
  {
     if (state == CLOCK)  // Dim LEDs for clock
     {
        if ((off_counter % 3) == 0)
           draw_buffer(fb + row_count);
     } else {
        if (state != RIDER) 
           draw_buffer(fb + row_count); // draw the buffer every interrupt         
        else 
           draw_rider(RiderPosition[rider_step], RiderTable[RiderStates[rider_step]][off_counter % 8]);
     }
  }

  if ((state == RIDER) && ((off_counter % 16) == 0)) {
    rider_step++;
    if (rider_step > 9)
      rider_step = 0;
  }

  SRTISC |= 0x40;  // clear RTIF
}


void MTIM_ISR (void)  // note: to change the scroll speed, change the MTIM period (default = 90ms)
{
  if (scroll_flag) // if we want to scroll...
  {
    if (ScrollDirection == 0) 
    {
      if (fb[row_count + 20] != 0x80)  // if we haven't reached the end of the message, keep on truckin'!
         row_count++;   // increment the row counter for scrolling purposes
      else
         if ((Effect & 0x1) && (state == SCROLL))
            ScrollDirection = 1;
         else
            row_count = 0;
    } else {
      if (row_count > 0)
         row_count--;
      else
         ScrollDirection = 0;
    }
  }

  MTIMSC_TOF = 0;   // Clear interrupt flag
}
     

void enable_scrolling (void)
{
  scroll_flag = 1;
  row_count = 0;
  ScrollDirection = 0;
}

void disable_scrolling (void)
{
  scroll_flag = 0;
  row_count = 0;
}


void change_state (void)
{
  disable_scrolling();
  clear_matrix();

  switch (state) 
  {
    case SLEEP:
  	  state = SCROLL;
  		break;
    case SCROLL:
      state = CLOCK;
      clear_matrix();
      clock_refresh = 1;
      break;
    case CLOCK:
      state = RIDER;
      break;
    case RIDER:
     	state = SET_MSG;
     	break;
    case SET_MSG:
     	state = SET_CLOCK;
     	break;
    case SET_CLOCK:
      state = SET_SPEED;
      break;
    case SET_SPEED:
     	state = SET_BOUNCE;
     	break;
    case SET_BOUNCE:
      state = SET_BLINK;
      break;
    case SET_BLINK:
      state = POV;
      break;
    case POV:
  		state = SLEEP;
  	break;
  }
   
	state_change_flag = 1;
}


////   Library for a 74595 Expanded Output Chip                        ////
////   from CCS PIC-C 74595.C driver and modified to not erase input   ////
////   byte when shifting out data.                                    ////
////                                                                   ////
////   write_expanded_outputs(eo);  Writes the array eo to the chips   ////
////                                                                   ////
void write_expanded_outputs (unsigned char eo[NUMBER_OF_74595]) 
{
    unsigned char i, j;

    REG_CLK = LOW;
    REG_STROBE = LOW;

    for (i = 0; i < NUMBER_OF_74595; ++i)  // start with LSB
    {
      for(j = 1; j <= 8; ++j) 
      {  
        // Clock out bits from the eo array (start with LSB)
        if ((eo[i] & 0x01) == 0)
            REG_DATA = LOW;
        else
            REG_DATA = HIGH;
   
        eo[i] >>= 1;
         
        REG_CLK = HIGH;
        REG_CLK = LOW;
      }
    }
    
    REG_STROBE = HIGH;
    REG_STROBE = LOW;
}

/**************************************************************
*	Function: 	delay
*	Parameters: 16-bit delay value
*	Return:		none
*
*	Simple delay loop.
**************************************************************/
 
void delay(unsigned int count) 
{
	unsigned int i;
	for (i = 0; i < count; i++);
}

// delay the specified number of milliseconds
// not really very accurate because it could get interrupted
void delay_ms(unsigned int ms)  
{
  unsigned int timer;

  while (ms != 0) {
    // this number is dependent on the clock frequency (currently at 16MHz)
    for (timer=0; timer <= 376; timer++);
    ms--;
  }
}


/*-------------------------- UNUSED BITS & PIECES --------------------------*/

// Freescale MMA7260QT 3-axis Accelerometer code 
/*  
    #define TRIAXSLEEP    PTBD_PTBD7    // Output
    #define THRESH        300

    // set ideal 0g and 1g calibration values
    #define X0g 512
    #define Y0g 512
    #define Z0g 512
      
    #define X1g 758
    #define Y1g 758
    #define Z1g 758
      
    // A/D channels
    #define X_AXIS        0x02
    #define Y_AXIS        0x01
    #define Z_AXIS        0x00
  
    ADCSC1 = X_AXIS;//read X channel
    while(!ADCSC1_COCO);
    xx = ADCR;

    delay(10);

    ADCSC1 = Y_AXIS;//read Y channel
    while(!ADCSC1_COCO);
    yy = ADCR;

    delay(10);
      
    ADCSC1 = Z_AXIS;//read Z channel
    while(!ADCSC1_COCO);
    zz = ADCR;
             
    TRIAXSLEEP = 0;   // go to sleep for TRIAX
      
    xx = xx - xx_cal;
    yy = yy - yy_cal;
    zz = zz - zz_cal;
    (void)printf ("X: %d  Y: %d  Z: %d\n\r", xx, yy, zz);
*/

